@@ -200,7 +200,7 @@ def nodeset():
         print("WARNING: option group source \"%s\" ignored"
               % options.groupsource, file=sys.stderr)
 
-    # We want -s <groupsource> to act as a substition of default groupsource
+    # We want -s <groupsource> to act as a substitution of default groupsource
     # (ie. it's not necessary to prefix group names by this group source).
     if options.groupsource:
         group_resolver.default_source_name = options.groupsource
@@ -287,7 +287,8 @@ def nodeset():
     if options.axis:
         if not options.axis.startswith('-'):
             # axis are 1-indexed in nodeset CLI (0 ignored)
-            xset.fold_axis = tuple(x-1 for x in RangeSet(options.axis) if x > 0)
+            xset.fold_axis = tuple(x-1 for x in \
+                                   RangeSet(options.axis).intiter() if x > 0)
         else:
             # negative axis index (only single number supported)
             xset.fold_axis = [int(options.axis)]
